@@ -34,16 +34,6 @@ typedef enum
 				 * statement */
 } CONN_Status;
 
-enum
-{
-	DISALLOW_UPDATABLE_CURSORS = 0,	/* No cursors are updatable */
-	ALLOW_STATIC_CURSORS = 1L, /* Static cursors are updatable */
-	ALLOW_KEYSET_DRIVEN_CURSORS = (1L << 1), /* Keyset-driven cursors are updatable */
-	ALLOW_DYNAMIC_CURSORS = (1L << 2), /* Dynamic cursors are updatable */
-	ALLOW_BULK_OPERATIONS = (1L << 3), /* Bulk operations available */
-	SENSE_SELF_OPERATIONS = (1L << 4), /* Sense self update/delete/add */
-};
-
 /*	These errors have general sql error state */
 #define CONNECTION_SERVER_NOT_REACHED				101
 #define CONNECTION_MSG_TOO_LONG					103
@@ -57,6 +47,7 @@ enum
 #define CONNECTION_NEED_PASSWORD				112
 #define CONNECTION_COMMUNICATION_ERROR				113
 
+#define CONN_ERROR_IGNORED						(-3)
 #define CONN_TRUNCATED							(-2)
 #define CONN_OPTION_VALUE_CHANGED					(-1)
 /*	These errors correspond to specific SQL states */
@@ -315,6 +306,19 @@ struct ConnectionClass_
 	char		unicode;
 	char		result_uncommitted;
 	char		lo_is_domain;
+	char		current_schema_valid;	/* is current_schema valid? TRUE when
+						 * current_schema == NULL means it's
+						 * really NULL, while FALSE means it's
+						 * unknown */
+	unsigned char	on_commit_in_progress;
+	/* for per statement rollback */
+	char		internal_svp;		/* is set? */
+	char		internal_op;		/* operation being executed as to internal savepoint */
+	unsigned char	lock_CC_for_rb;
+	unsigned char	rbonerr;
+	unsigned char	opt_in_progress;
+	unsigned char	opt_previous;
+
 	char		*original_client_encoding;
 	char		*locale_encoding;
 	char		*server_encoding;
@@ -323,10 +327,6 @@ struct ConnectionClass_
 	SQLUINTEGER	isolation;		/* isolation level initially unknown */
 	SQLUINTEGER	server_isolation;	/* isolation at server initially unknown */
 	char		*current_schema;
-	char		current_schema_valid;	/* is current_schema valid? TRUE when
-										 * current_schema == NULL means it's
-										 * really NULL, while FALSE means it's
-										 * unknown */
 	StatementClass *unnamed_prepared_stmt;
 	Int2		max_identifier_length;
 	Int2		num_discardp;
@@ -384,7 +384,16 @@ enum {
 #define	CC_set_dtc_isolated(x)	((x)->gTranInfo |= DTC_ISOLATED)
 #define	CC_is_idle_in_global_transaction(x)	(0 != ((x)->gTranInfo & DTC_PREPARE_REQUESTED) || (x)->gTranInfo == DTC_IN_PROGRESS)
 #endif /* _HANDLE_ENLIST_IN_DTC_ */
-
+/* statement callback */
+#define CC_start_stmt(a)        ((a)->rbonerr = 0)
+#define CC_start_tc_stmt(a)     ((a)->rbonerr = (1L << 1))
+#define CC_is_tc_stmt(a)        (((a)->rbonerr & (1L << 1)) != 0)
+#define CC_start_rb_stmt(a)     ((a)->rbonerr = (1L << 2))
+#define CC_is_rb_stmt(a)        (((a)->rbonerr & (1L << 2)) != 0)
+#define CC_set_accessed_db(a)   ((a)->rbonerr |= (1L << 3))
+#define CC_accessed_db(a)       (((a)->rbonerr & (1L << 3)) != 0)
+#define CC_start_rbpoint(a)     ((a)->rbonerr |= (1L << 4), (a)->internal_svp = 1)
+#define CC_started_rbpoint(a)   (((a)->rbonerr & (1L << 4)) != 0)
 
 /*	prototypes */
 ConnectionClass *CC_Constructor(void);
@@ -432,9 +441,9 @@ const		char *CurrCatString(const ConnectionClass *self);
 SQLUINTEGER	CC_get_isolation(ConnectionClass *self);
 
 SQLCHAR	*make_lstring_ifneeded(ConnectionClass *, const SQLCHAR *s, ssize_t len, BOOL);
-char	*schema_strcat(char *buf, const char *fmt, const SQLCHAR *s, SQLLEN len,
+char	*schema_strcat(char *buf, int buflen, const char *fmt, const SQLCHAR *s, SQLLEN len,
 		const SQLCHAR *, SQLLEN, ConnectionClass *conn);
-char	*schema_strcat1(char *buf, const char *fmt, const char *s1,
+char	*schema_strcat1(char *buf, int buflen, const char *fmt, const char *s1,
 				const char *s,
 				const SQLCHAR *, int, ConnectionClass *conn);
 
@@ -451,11 +460,47 @@ enum {
 	,GO_INTO_TRANSACTION	= (1L << 2) /* issue BEGIN in advance */
 	,ROLLBACK_ON_ERROR	= (1L << 3) /* rollback the query when an error occurs */
 	,END_WITH_COMMIT	= (1L << 4) /* the query ends with COMMIT command */
+	,READ_ONLY_QUERY	= (1L << 5) /* the query is read-only */
 };
 /* CC_on_abort options */
 #define	NO_TRANS		1L
 #define	CONN_DEAD		(1L << 1) /* connection is no longer valid */
 
+/*
+ *	internal savepoint related
+ */
+
+#define	_RELEASE_INTERNAL_SAVEPOINT
+
+/*      Internal rollback */
+enum {
+	PER_STATEMENT_ROLLBACK = 1
+	,PER_QUERY_ROLLBACK
+};
+int CC_internal_rollback(ConnectionClass *, int rollback_type, BOOL ignore_abort);
+
+/*     Commands generated */
+enum {
+	INTERNAL_SAVEPOINT_OPERATION = 1
+	,INTERNAL_ROLLBACK_OPERATION
+};
+int	GenerateSvpCommand(ConnectionClass *conn, int type, char *cmd, int bufsize);
+
+/*      Operations in progress */
+enum {
+        SAVEPOINT_IN_PROGRESS = 1
+        ,PREPEND_IN_PROGRESS
+};
+/*      StatementSvp entry option */
+enum {
+        SVPOPT_RDONLY = 1L
+        ,SVPOPT_REDUCE_ROUNDTRIP = (1L << 1)
+};
+#define	INIT_SVPOPT	(SVPOPT_RDONLY)
+#define CC_svp_init(a) ((a)->internal_svp = (a)->internal_op = 0, (a)->opt_in_progress = (a)->opt_previous = INIT_SVPOPT)
+#define CC_init_opt_in_progress(a) ((a)->opt_in_progress = INIT_SVPOPT)
+#define CC_init_opt_previous(a) ((a)->opt_previous = INIT_SVPOPT)
+
 #ifdef	__cplusplus
 }
 #endif
