@@ -30,11 +30,10 @@
 #include "connexp.h"
 #endif /* _HANDLE_ENLIST_IN_DTC_ */
 
-extern GLOBAL_VALUES globals;
 
 extern HINSTANCE s_hModule;
 static int	driver_optionsDraw(HWND, const ConnInfo *, int src, BOOL enable);
-static int	driver_options_update(HWND hdlg, ConnInfo *ci, const char *);
+static int	driver_options_update(HWND hdlg, ConnInfo *ci);
 
 static int	ds_options_update(HWND hdlg, ConnInfo *ci);
 
@@ -124,55 +123,53 @@ GetDlgStuff(HWND hdlg, ConnInfo *ci)
 	STR_TO_NAME(ci->password, medium_buf);
 	GetDlgItemText(hdlg, IDC_PORT, ci->port, sizeof(ci->port));
 	sslposition = (int)(DWORD)SendMessage(GetDlgItem(hdlg, IDC_SSLMODE), CB_GETCURSEL, 0L, 0L);
-	strncpy_null(ci->sslmode, modetab[sslposition].modestr, sizeof(ci->sslmode));
+	STRCPY_FIXED(ci->sslmode, modetab[sslposition].modestr);
 }
 
+static void
+getDriversDefaultsOfCi(const ConnInfo *ci, GLOBAL_VALUES *glbv)
+{
+	const char *drivername = NULL;
+
+	if (ci->drivername[0])
+		drivername = ci->drivername;
+	else if (NAME_IS_VALID(ci->drivers.drivername))
+		drivername = SAFE_NAME(ci->drivers.drivername);	
+	if (drivername && drivername[0])
+		getDriversDefaults(drivername, glbv);
+	else
+		getDriversDefaults(INVALID_DRIVER, glbv);
+}
 
 static int
 driver_optionsDraw(HWND hdlg, const ConnInfo *ci, int src, BOOL enable)
 {
 	const GLOBAL_VALUES *comval = NULL;
-	static BOOL defset = FALSE;
-	static GLOBAL_VALUES defval;
+	const char * drivername = NULL;
+	GLOBAL_VALUES defval;
 
+mylog("!!!! %s;%d in\n", __FUNCTION__, src);
+	init_globals(&defval);
 	switch (src)
 	{
-		case 0:			/* driver common */
-			comval = &globals;
+		case 0:			/* default */
+			getDriversDefaultsOfCi(ci, &defval);
+			defval.debug = DEFAULT_DEBUG;
+			defval.commlog = DEFAULT_COMMLOG;
+			comval = &defval;
 			break;
 		case 1:			/* dsn specific */
 			comval = &(ci->drivers);
 			break;
-		case 2:			/* default */
-			if (!defset)
-			{
-				defval.commlog = DEFAULT_COMMLOG;
-				defval.unique_index = DEFAULT_UNIQUEINDEX;
-				defval.onlyread = DEFAULT_READONLY;
-				defval.use_declarefetch = DEFAULT_USEDECLAREFETCH;
-
-				defval.parse = DEFAULT_PARSE;
-				defval.debug = DEFAULT_DEBUG;
-
-				/* Unknown Sizes */
-				defval.unknown_sizes = DEFAULT_UNKNOWNSIZES;
-				defval.text_as_longvarchar = DEFAULT_TEXTASLONGVARCHAR;
-				defval.unknowns_as_longvarchar = DEFAULT_UNKNOWNSASLONGVARCHAR;
-				defval.bools_as_char = DEFAULT_BOOLSASCHAR;
-			}
-			defset = TRUE;
-			comval = &defval;
-			break;
 		default:
 			return 0;
 	}
 
 	ShowWindow(GetDlgItem(hdlg, DRV_MSG_LABEL2), enable ? SW_SHOW : SW_HIDE);
-	CheckDlgButton(hdlg, DRV_COMMLOG, comval->commlog);
+	CheckDlgButton(hdlg, DRV_COMMLOG, comval->commlog > 0);
 	CheckDlgButton(hdlg, DRV_UNIQUEINDEX, comval->unique_index);
 	/* EnableWindow(GetDlgItem(hdlg, DRV_UNIQUEINDEX), enable); */
-	CheckDlgButton(hdlg, DRV_READONLY, comval->onlyread);
-	EnableWindow(GetDlgItem(hdlg, DRV_READONLY), enable);
+	EnableWindow(GetDlgItem(hdlg, DRV_READONLY), FALSE);
 	CheckDlgButton(hdlg, DRV_USEDECLAREFETCH, comval->use_declarefetch);
 
 	/* Unknown Sizes clear */
@@ -198,35 +195,31 @@ driver_optionsDraw(HWND hdlg, const ConnInfo *ci, int src, BOOL enable)
 	CheckDlgButton(hdlg, DRV_UNKNOWNS_LONGVARCHAR, comval->unknowns_as_longvarchar);
 	CheckDlgButton(hdlg, DRV_BOOLS_CHAR, comval->bools_as_char);
 	CheckDlgButton(hdlg, DRV_PARSE, comval->parse);
-	CheckDlgButton(hdlg, DRV_DEBUG, comval->debug);
+	CheckDlgButton(hdlg, DRV_DEBUG, comval->debug > 0);
 	SetDlgItemInt(hdlg, DRV_CACHE_SIZE, comval->fetch_max, FALSE);
 	SetDlgItemInt(hdlg, DRV_VARCHAR_SIZE, comval->max_varchar_size, FALSE);
 	SetDlgItemInt(hdlg, DRV_LONGVARCHAR_SIZE, comval->max_longvarchar_size, TRUE);
 	SetDlgItemText(hdlg, DRV_EXTRASYSTABLEPREFIXES, comval->extra_systable_prefixes);
 
 	/* Driver Connection Settings */
-	SetDlgItemText(hdlg, DRV_CONNSETTINGS, SAFE_NAME(comval->conn_settings));
-	EnableWindow(GetDlgItem(hdlg, DRV_CONNSETTINGS), enable);
+	EnableWindow(GetDlgItem(hdlg, DRV_CONNSETTINGS), FALSE);
 	ShowWindow(GetDlgItem(hdlg, ID2NDPAGE), enable ? SW_HIDE : SW_SHOW);
 	ShowWindow(GetDlgItem(hdlg, ID3RDPAGE), enable ? SW_HIDE : SW_SHOW);
+
+	finalize_globals(&defval);
 	return 0;
 }
 
 static int
-driver_options_update(HWND hdlg, ConnInfo *ci, const char *updateDriver)
+driver_options_update(HWND hdlg, ConnInfo *ci)
 {
 	GLOBAL_VALUES *comval;
 
-	if (ci)
-		comval = &(ci->drivers);
-	else
-		comval = &globals;
+// mylog("!!!! %s in\n", __FUNCTION__);
+	comval = &(ci->drivers);
+
 	comval->commlog = IsDlgButtonChecked(hdlg, DRV_COMMLOG);
 	comval->unique_index = IsDlgButtonChecked(hdlg, DRV_UNIQUEINDEX);
-	if (!ci)
-	{
-		comval->onlyread = IsDlgButtonChecked(hdlg, DRV_READONLY);
-	}
 	comval->use_declarefetch = IsDlgButtonChecked(hdlg, DRV_USEDECLAREFETCH);
 
 	/* Unknown (Default) Data Type sizes */
@@ -254,69 +247,10 @@ driver_options_update(HWND hdlg, ConnInfo *ci, const char *updateDriver)
 
 	GetDlgItemText(hdlg, DRV_EXTRASYSTABLEPREFIXES, comval->extra_systable_prefixes, sizeof(comval->extra_systable_prefixes));
 
-	/* Driver Connection Settings */
-	if (!ci)
-	{
-		char	conn_settings[LARGE_REGISTRY_LEN];
-
-		GetDlgItemText(hdlg, DRV_CONNSETTINGS, conn_settings, sizeof(conn_settings));
-		if ('\0' != conn_settings[0])
-			STR_TO_NAME(comval->conn_settings, conn_settings);
-	}
-
-	if (updateDriver)
-	{
-		if (writeDriverCommoninfo(ODBCINST_INI, updateDriver, comval) < 0)
-			MessageBox(hdlg, "impossible to update the values, sorry", "Update Error", MB_ICONEXCLAMATION | MB_OK);
-;
-	}
-
 	/* fall through */
 	return 0;
 }
 
-LRESULT		CALLBACK
-driver_optionsProc(HWND hdlg,
-				   UINT wMsg,
-				   WPARAM wParam,
-				   LPARAM lParam)
-{
-	ConnInfo   *ci;
-	char	strbuf[128];
-
-	switch (wMsg)
-	{
-		case WM_INITDIALOG:
-			SetWindowLongPtr(hdlg, DWLP_USER, lParam);		/* save for OK etc */
-			ci = (ConnInfo *) lParam;
-			LoadString(s_hModule, IDS_ADVANCE_OPTION_DEF, strbuf, sizeof(strbuf));
-			SetWindowText(hdlg, strbuf);
-			LoadString(s_hModule, IDS_ADVANCE_SAVE, strbuf, sizeof(strbuf));
-			SetWindowText(GetDlgItem(hdlg, IDOK), strbuf);
-			ShowWindow(GetDlgItem(hdlg, IDAPPLY), SW_HIDE);
-			driver_optionsDraw(hdlg, ci, 0, TRUE);
-			break;
-
-		case WM_COMMAND:
-			switch (GET_WM_COMMAND_ID(wParam, lParam))
-			{
-				case IDOK:
-					ci = (ConnInfo *) GetWindowLongPtr(hdlg, DWLP_USER);
-					driver_options_update(hdlg, NULL,
-						ci ? ci->drivername : NULL);
-
-				case IDCANCEL:
-					EndDialog(hdlg, GET_WM_COMMAND_ID(wParam, lParam) == IDOK);
-					return TRUE;
-
-				case IDDEFAULTS:
-					driver_optionsDraw(hdlg, NULL, 2, TRUE);
-					break;
-			}
-	}
-
-	return FALSE;
-}
 
 #ifdef _HANDLE_ENLIST_IN_DTC_
 static
@@ -369,13 +303,19 @@ global_optionsProc(HWND hdlg,
 	ConnInfo	*ci;
 	char logdir[PATH_MAX];
 	const char *logmsg;
+	GLOBAL_VALUES	defval;
 
+// if (WM_INITDIALOG == wMsg || WM_COMMAND == wMsg)
+// mylog("!!!! %s:%d in\n", __FUNCTION__, wMsg);
+	init_globals(&defval);
 	switch (wMsg)
 	{
 		case WM_INITDIALOG:
 			SetWindowLongPtr(hdlg, DWLP_USER, lParam); /* save for test etc */ 
-			CheckDlgButton(hdlg, DRV_COMMLOG, globals.commlog);
-			CheckDlgButton(hdlg, DRV_DEBUG, globals.debug);
+			ci = (ConnInfo *) lParam;
+			getDriversDefaultsOfCi(ci, &defval);
+			CheckDlgButton(hdlg, DRV_COMMLOG, getGlobalCommlog());
+			CheckDlgButton(hdlg, DRV_DEBUG, getGlobalDebug());
 			getLogDir(logdir, sizeof(logdir));
 			SetDlgItemText(hdlg, DS_LOGDIR, logdir);
 #ifdef _HANDLE_ENLIST_IN_DTC_
@@ -399,15 +339,17 @@ global_optionsProc(HWND hdlg,
 			switch (GET_WM_COMMAND_ID(wParam, lParam))
 			{
 				case IDOK:
+					getDriversDefaultsOfCi(ci, &defval);
 					GetDlgItemText(hdlg, DS_LOGDIR, logdir, sizeof(logdir));
 					if (logdir[0] && (logmsg = IsAnExistingDirectory(logdir)) != NULL)
 					{
 						MessageBox(hdlg, "Folder for logging error", logmsg, MB_ICONEXCLAMATION | MB_OK);
 						break;
 					}
-					globals.commlog = IsDlgButtonChecked(hdlg, DRV_COMMLOG);
-					globals.debug = IsDlgButtonChecked(hdlg, DRV_DEBUG);
-					if (writeDriverCommoninfo(ODBCINST_INI, ci->drivername, &globals) < 0)
+					setGlobalCommlog(IsDlgButtonChecked(hdlg, DRV_COMMLOG));
+					setGlobalDebug(IsDlgButtonChecked(hdlg, DRV_DEBUG));
+					writeGlobalLogs();
+					if (writeDriversDefaults(ci->drivername, &defval) < 0)
 						MessageBox(hdlg, "Sorry, impossible to update the values\nWrite permission seems to be needed", "Update Error", MB_ICONEXCLAMATION | MB_OK);
 					setLogDir(logdir[0] ? logdir : NULL);
 #ifdef _HANDLE_ENLIST_IN_DTC_
@@ -424,6 +366,7 @@ global_optionsProc(HWND hdlg,
 			}
 	}
 
+	finalize_globals(&defval);
 	return FALSE;
 }
 
@@ -436,6 +379,8 @@ ds_options1Proc(HWND hdlg,
 	ConnInfo   *ci;
 	char	strbuf[128];
 
+// if (WM_INITDIALOG == wMsg || WM_COMMAND == wMsg)
+// mylog("!!!! %s:%d in\n", __FUNCTION__, wMsg);
 	switch (wMsg)
 	{
 		case WM_INITDIALOG:
@@ -451,8 +396,8 @@ ds_options1Proc(HWND hdlg,
 						fbuf,
 						sizeof(fbuf));
 				if (cmd <= 0)
-					strcpy(fbuf, "Advanced Options (%s) 1/3");
-				sprintf(strbuf, fbuf, ci->dsn);
+					STRCPY_FIXED(fbuf, "Advanced Options (%s) 1/3");
+				SPRINTF_FIXED(strbuf, fbuf, ci->dsn);
 				SetWindowText(hdlg, strbuf);
 			}
 			else
@@ -469,14 +414,14 @@ ds_options1Proc(HWND hdlg,
 			switch (GET_WM_COMMAND_ID(wParam, lParam))
 			{
 				case IDOK:
-					driver_options_update(hdlg, ci, NULL);
+					driver_options_update(hdlg, ci);
 
 				case IDCANCEL:
 					EndDialog(hdlg, GET_WM_COMMAND_ID(wParam, lParam) == IDOK);
 					return TRUE;
 
 				case IDAPPLY:
-					driver_options_update(hdlg, ci, NULL);
+					driver_options_update(hdlg, ci);
 					SendMessage(GetWindow(hdlg, GW_OWNER), WM_COMMAND, wParam, lParam);
 					break;
 
@@ -485,14 +430,14 @@ ds_options1Proc(HWND hdlg,
 					break;
 
 				case ID2NDPAGE:
-					driver_options_update(hdlg, ci, NULL);
+					driver_options_update(hdlg, ci);
 					EndDialog(hdlg, FALSE);
 					DialogBoxParam(s_hModule,
 								   MAKEINTRESOURCE(DLG_OPTIONS_DS),
 								   hdlg, ds_options2Proc, (LPARAM) ci);
 					break;
 				case ID3RDPAGE:
-					driver_options_update(hdlg, ci, NULL);
+					driver_options_update(hdlg, ci);
 					EndDialog(hdlg, FALSE);
 					DialogBoxParam(s_hModule,
 								   MAKEINTRESOURCE(DLG_OPTIONS_DS3),
@@ -512,7 +457,7 @@ ds_options_update(HWND hdlg, ConnInfo *ci)
 	mylog("%s: got ci = %p\n", __FUNCTION__, ci);
 
 	/* Readonly */
-	sprintf(ci->onlyread, "%d", IsDlgButtonChecked(hdlg, DS_READONLY));
+	ITOA_FIXED(ci->onlyread, IsDlgButtonChecked(hdlg, DS_READONLY));
 
 	/* Issue rollback command on error */
 	if (IsDlgButtonChecked(hdlg, DS_NO_ROLLBACK))
@@ -541,9 +486,9 @@ ds_options_update(HWND hdlg, ConnInfo *ci)
 
 	GetDlgItemText(hdlg, DS_EXTRA_OPTIONS, buf, sizeof(buf));
 	setExtraOptions(ci, buf, NULL);
-	sprintf(ci->show_system_tables, "%d", IsDlgButtonChecked(hdlg, DS_SHOWSYSTEMTABLES));
+	ITOA_FIXED(ci->show_system_tables, IsDlgButtonChecked(hdlg, DS_SHOWSYSTEMTABLES));
 
-	sprintf(ci->row_versioning, "%d", IsDlgButtonChecked(hdlg, DS_ROWVERSIONING));
+	ITOA_FIXED(ci->row_versioning, IsDlgButtonChecked(hdlg, DS_ROWVERSIONING));
 	ci->lf_conversion = IsDlgButtonChecked(hdlg, DS_LFCONVERSION);
 	ci->true_is_minus1 = IsDlgButtonChecked(hdlg, DS_TRUEISMINUS1);
 	ci->allow_keyset = IsDlgButtonChecked(hdlg, DS_UPDATABLECURSORS);
@@ -553,8 +498,8 @@ ds_options_update(HWND hdlg, ConnInfo *ci)
 	ci->gssauth_use_gssapi = IsDlgButtonChecked(hdlg, DS_GSSAUTHUSEGSSAPI);
 
 	/* OID Options */
-	sprintf(ci->fake_oid_index, "%d", IsDlgButtonChecked(hdlg, DS_FAKEOIDINDEX));
-	sprintf(ci->show_oid_column, "%d", IsDlgButtonChecked(hdlg, DS_SHOWOIDCOLUMN));
+	ITOA_FIXED(ci->fake_oid_index, IsDlgButtonChecked(hdlg, DS_FAKEOIDINDEX));
+	ITOA_FIXED(ci->show_oid_column, IsDlgButtonChecked(hdlg, DS_SHOWOIDCOLUMN));
 
 	/* Datasource Connection Settings */
 	{
@@ -599,6 +544,8 @@ ds_options2Proc(HWND hdlg,
 	DWORD		cmd;
 	BOOL		enable;
 
+// if (WM_INITDIALOG == wMsg || WM_COMMAND == wMsg)
+// mylog("!!!! %s:%d in\n", __FUNCTION__, wMsg);
 	switch (wMsg)
 	{
 		case WM_INITDIALOG:
@@ -615,8 +562,8 @@ ds_options2Proc(HWND hdlg,
 						fbuf,
 						sizeof(fbuf));
 				if (cmd <= 0)
-					strcpy(fbuf, "Advanced Options (%s) 2/3");
-				sprintf(buf, fbuf, ci->dsn);
+					STRCPY_FIXED(fbuf, "Advanced Options (%s) 2/3");
+				SPRINTF_FIXED(buf, fbuf, ci->dsn);
 				SetWindowText(hdlg, buf);
 			}
 			else
@@ -666,7 +613,7 @@ ds_options2Proc(HWND hdlg,
 				default:
 					CheckDlgButton(hdlg, DS_INT8_AS_DEFAULT, 1);
 			}
-			sprintf(buf, "0x%x", getExtraOptions(ci));
+			SPRINTF_FIXED(buf, "0x%x", getExtraOptions(ci));
 			SetDlgItemText(hdlg, DS_EXTRA_OPTIONS, buf);
 
 			CheckDlgButton(hdlg, DS_SHOWOIDCOLUMN, atoi(ci->show_oid_column));
@@ -692,12 +639,12 @@ ds_options2Proc(HWND hdlg,
 			{
 				if (ci->keepalive_idle > 0)
 				{
-					snprintf(buf, sizeof(buf), "%d", ci->keepalive_idle);
+					ITOA_FIXED(buf, ci->keepalive_idle);
 					SetDlgItemText(hdlg, DS_KEEPALIVETIME, buf);
 				}
 				if (ci->keepalive_interval > 0)
 				{
-					snprintf(buf, sizeof(buf), "%d", ci->keepalive_interval);
+					ITOA_FIXED(buf, ci->keepalive_interval);
 					SetDlgItemText(hdlg, DS_KEEPALIVEINTERVAL, buf);
 				}
 			}
@@ -752,6 +699,7 @@ ds_options3Draw(HWND hdlg, const ConnInfo *ci)
 	BOOL	enable = TRUE;
 	static BOOL defset = FALSE;
 
+mylog("!!!! %s in\n", __FUNCTION__);
 #ifdef	_HANDLE_ENLIST_IN_DTC_
 	switch (ci->xa_opt)
 	{
@@ -846,6 +794,8 @@ ds_options3Proc(HWND hdlg,
 	char		buf[128];
 	DWORD		cmd;
 
+if (WM_INITDIALOG == wMsg || WM_COMMAND == wMsg)
+mylog("!!!! %s:%d in\n", __FUNCTION__, wMsg);
 	switch (wMsg)
 	{
 		case WM_INITDIALOG:
@@ -862,8 +812,8 @@ ds_options3Proc(HWND hdlg,
 						fbuf,
 						sizeof(fbuf));
 				if (cmd <= 0)
-					strcpy(fbuf, "Advanced Options (%s) 3/3");
-				sprintf(buf, fbuf, ci->dsn);
+					STRCPY_FIXED(fbuf, "Advanced Options (%s) 3/3");
+				SPRINTF_FIXED(buf, fbuf, ci->dsn);
 				SetWindowText(hdlg, buf);
 			}
 			else
@@ -889,9 +839,10 @@ ds_options3Proc(HWND hdlg,
 					SendMessage(GetWindow(hdlg, GW_OWNER), WM_COMMAND, wParam, lParam);
 					break;
 				case IDC_TEST:
-					memcpy(&tmpInfo, ci, sizeof(tmpInfo));
+					CC_copy_conninfo(&tmpInfo, ci);
 					ds_options3_update(hdlg, &tmpInfo);
 					test_connection(hdlg, &tmpInfo, TRUE);
+					CC_conninfo_release(&tmpInfo);
 					break;
 				case ID1STPAGE:
 					ds_options3_update(hdlg, ci);
