@@ -17,6 +17,7 @@
 
 #include "loadlib.h"
 #include "pgenlist.h"
+#include "misc.h"
 
 #ifdef  WIN32
 #ifdef  _MSC_VER
@@ -112,7 +113,7 @@ HMODULE MODULE_load_from_psqlodbc_path(const char *module_name)
 
 		_splitpath(szFileName, drive, dir, NULL, NULL);
 		GetSystemDirectory(sysdir, MAX_PATH);
-		snprintf(szFileName, sizeof(szFileName), "%s%s%s.dll", drive, dir, module_name);
+		SPRINTF_FIXED(szFileName, "%s%s%s.dll", drive, dir, module_name);
 		if (_strnicmp(szFileName, sysdir, strlen(sysdir)) != 0)
 		{
 			hmodule = LoadLibraryEx(szFileName, NULL, LOAD_WITH_ALTERED_SEARCH_PATH);
@@ -180,18 +181,20 @@ void CleanupDelayLoadedDLLs(void)
 	{
 		if (enlist_module != NULL)
 		{
+			mylog("Freeing Library %s\n", pgenlistdll);
 			FreeLibrary(enlist_module);
-			mylog("FreeLibrary %s\n", pgenlistdll);
 		}
+		mylog("%s unloading\n", pgenlistdll);
 		success = (*func)(pgenlistdll);
-		mylog("%s unload success=%d\n", pgenlistdll, success);
+		mylog("%s unloaded success=%d\n", pgenlistdll, success);
 		loaded_pgenlist = FALSE;
 	}
 	if (loaded_psqlodbc)
 	{
+		mylog("%s unloading\n", psqlodbcdll);
 		success = (*func)(psqlodbcdll);
+		mylog("%s unloaded success=%d\n", psqlodbcdll, success);
 		loaded_psqlodbc = FALSE;
-		mylog("%s unload success=%d\n", psqlodbcdll, success);
 	}
 	return;
 }
