@@ -15,6 +15,7 @@
 #define	_MYLOG_FUNCS_IMPLEMENT_
 #include "psqlodbc.h"
 #include "dlg_specific.h"
+#include "misc.h"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -50,12 +51,11 @@
 #define PG_BINARY_A			"a"
 #endif /* WIN32 */
 
-extern GLOBAL_VALUES globals;
 
 static char *logdir = NULL;
 
 void
-generate_filename(const char *dirname, const char *prefix, char *filename)
+generate_filename(const char *dirname, const char *prefix, char *filename, size_t filenamelen)
 {
 #ifdef	WIN32
 	int	pid;
@@ -71,20 +71,19 @@ generate_filename(const char *dirname, const char *prefix, char *filename)
 	if (dirname == 0 || filename == 0)
 		return;
 
-	strcpy(filename, dirname);
-	strcat(filename, DIRSEPARATOR);
+	snprintf(filename, filenamelen, "%s%s", dirname, DIRSEPARATOR);
 	if (prefix != 0)
-		strcat(filename, prefix);
+		strlcat(filename, prefix, filenamelen);
 #ifndef WIN32
 	if (ptr)
-		strcat(filename, ptr->pw_name);
+		strlcat(filename, ptr->pw_name, filenamelen);
 #endif
-	sprintf(filename, "%s%u%s", filename, pid, ".log");
+	snprintfcat(filename, filenamelen, "%u%s", pid, ".log");
 	return;
 }
 
 static void
-generate_homefile(const char *prefix, char *filename)
+generate_homefile(const char *prefix, char *filename, size_t filenamelen)
 {
 	char	dir[PATH_MAX];
 #ifdef	WIN32
@@ -92,13 +91,13 @@ generate_homefile(const char *prefix, char *filename)
 
 	dir[0] = '\0';
 	if (ptr=getenv("HOMEDRIVE"), NULL != ptr)
-		strcat(dir, ptr);
+		strlcat(dir, ptr, filenamelen);
 	if (ptr=getenv("HOMEPATH"), NULL != ptr)
-		strcat(dir, ptr);
+		strlcat(dir, ptr, filenamelen);
 #else
-	strcpy(dir, "~");
+	STRCPY_FIXED(dir, "~");
 #endif /* WIN32 */
-	generate_filename(dir, prefix, filename);
+	generate_filename(dir, prefix, filename, filenamelen);
 
 	return;
 }
@@ -186,8 +185,8 @@ logs_on_off(int cnopen, int mylog_onoff, int qlog_onoff)
 	}
 	else if (mylog_off_count > 0)
 		mylog_on = 0;
-	else if (globals.debug > 0)
-		mylog_on = globals.debug;
+	else if (getGlobalDebug() > 0)
+		mylog_on = getGlobalDebug();
 	if (qlog_onoff)
 		qlog_on_count += cnopen;
 	else
@@ -196,8 +195,8 @@ logs_on_off(int cnopen, int mylog_onoff, int qlog_onoff)
 		qlog_on = 1;
 	else if (qlog_off_count > 0)
 		qlog_on = 0;
-	else if (globals.commlog > 0)
-		qlog_on = globals.commlog;
+	else if (getGlobalCommlog() > 0)
+		qlog_on = getGlobalCommlog();
 	LEAVE_QLOG_CS;
 	LEAVE_MYLOG_CS;
 }
@@ -219,15 +218,15 @@ static void MLOG_open()
 
 	if (MLOGFP) return;
 
-	generate_filename(logdir ? logdir : MYLOGDIR, MYLOGFILE, filebuf);
+	generate_filename(logdir ? logdir : MYLOGDIR, MYLOGFILE, filebuf, sizeof(filebuf));
 	MLOGFP = fopen(filebuf, PG_BINARY_A);
 	if (!MLOGFP)
 	{
 		int lasterror = GENERAL_ERRNO;
  
 		open_error = TRUE;
-		snprintf(errbuf, sizeof(errbuf), "%s open error %d\n", filebuf, lasterror);
-		generate_homefile(MYLOGFILE, filebuf);
+		SPRINTF_FIXED(errbuf, "%s open error %d\n", filebuf, lasterror);
+		generate_homefile(MYLOGFILE, filebuf, sizeof(filebuf));
 		MLOGFP = fopen(filebuf, PG_BINARY_A);
 	}
 	if (MLOGFP)
@@ -317,11 +316,11 @@ qlog(char *fmt,...)
 
 	if (!QLOGFP)
 	{
-		generate_filename(logdir ? logdir : QLOGDIR, QLOGFILE, filebuf);
+		generate_filename(logdir ? logdir : QLOGDIR, QLOGFILE, filebuf, sizeof(filebuf));
 		QLOGFP = fopen(filebuf, PG_BINARY_A);
 		if (!QLOGFP)
 		{
-			generate_homefile(QLOGFILE, filebuf);
+			generate_homefile(QLOGFILE, filebuf, sizeof(filebuf));
 			QLOGFP = fopen(filebuf, PG_BINARY_A);
 		}
 		if (QLOGFP)
@@ -358,6 +357,94 @@ static void qlog_finalize(void)
 	DELETE_QLOG_CS;
 }
 
+static int	globalDebug = -1;
+int
+getGlobalDebug()
+{
+	char	temp[16];
+
+	if (globalDebug >=0)
+		return globalDebug;
+	/* Debug is stored in the driver section */
+	SQLGetPrivateProfileString(DBMS_NAME, INI_DEBUG, "", temp, sizeof(temp), ODBCINST_INI);
+	if (temp[0])
+		globalDebug = atoi(temp);
+	else
+		globalDebug = DEFAULT_DEBUG;
+
+	return globalDebug;
+}
+
+int
+setGlobalDebug(int val)
+{
+	return (globalDebug = val);
+}
+
+static int	globalCommlog = -1;
+int
+getGlobalCommlog()
+{
+	char	temp[16];
+
+	if (globalCommlog >= 0)
+		return globalCommlog;
+	/* Commlog is stored in the driver section */
+	SQLGetPrivateProfileString(DBMS_NAME, INI_COMMLOG, "", temp, sizeof(temp), ODBCINST_INI);
+	if (temp[0])
+		globalCommlog = atoi(temp);
+	else
+		globalCommlog = DEFAULT_COMMLOG;
+
+	return globalCommlog;
+}
+
+int
+setGlobalCommlog(int val)
+{
+	return (globalCommlog = val);
+}
+
+int
+writeGlobalLogs()
+{
+	char	temp[10];
+
+	ITOA_FIXED(temp, globalDebug);
+	SQLWritePrivateProfileString(DBMS_NAME, INI_DEBUG, temp, ODBCINST_INI);
+	ITOA_FIXED(temp, globalCommlog);
+	SQLWritePrivateProfileString(DBMS_NAME, INI_COMMLOG, temp, ODBCINST_INI);
+	return 0;
+}
+
+int
+getLogDir(char *dir, int dirmax)
+{
+	return SQLGetPrivateProfileString(DBMS_NAME, INI_LOGDIR, "", dir, dirmax, ODBCINST_INI);
+}
+
+int
+setLogDir(const char *dir)
+{
+	return SQLWritePrivateProfileString(DBMS_NAME, INI_LOGDIR, dir, ODBCINST_INI);
+}
+
+/*
+ *	This function starts a logging out of connections according the ODBCINST.INI
+ *	portion of the DBMS_NAME registry.
+ */
+static void
+start_logging()
+{
+	/*
+	 * GlobalDebug or GlobalCommlog means whether take mylog or commlog
+	 * out of the connection time or not but doesn't mean the default of
+	 * ci->drivers.debug(commlog).
+	 */
+	logs_on_off(0, 0, 0);
+	mylog("\t%s:Global.debug&commlog=%d&%d\n", __FUNCTION__, getGlobalDebug(), getGlobalCommlog());
+}
+
 void InitializeLogging(void)
 {
 	char dir[PATH_MAX];
@@ -367,6 +454,7 @@ void InitializeLogging(void)
 		logdir = strdup(dir);
 	mylog_initialize();
 	qlog_initialize();
+	start_logging();
 }
 
 void FinalizeLogging(void)
