@@ -36,6 +36,7 @@
 #include "catfunc.h"
 
 #include "multibyte.h"
+#include "misc.h"
 
 #define FLD_INCR	32
 #define TAB_INCR	8
@@ -390,10 +391,10 @@ static BOOL CheckHasOids(StatementClass * stmt)
 	if (!stmt->ti || !stmt->ti[0])
 		return FALSE;
 	ti = stmt->ti[0];
-	snprintf(query, sizeof(query),
+	SPRINTF_FIXED(query,
 			 "select relhasoids, c.oid, relhassubclass from pg_class c, pg_namespace n where relname = '%s' and nspname = '%s' and c.relnamespace = n.oid",
 			 SAFE_NAME(ti->table_name), SAFE_NAME(ti->schema_name));
-	res = CC_send_query(conn, query, NULL, ROLLBACK_ON_ERROR | IGNORE_ABORT_ON_CONN, NULL);
+	res = CC_send_query(conn, query, NULL, READ_ONLY_QUERY, NULL);
 	if (QR_command_maybe_successful(res))
 	{
 		stmt->num_key_fields = PG_NUM_NORMAL_KEYS;
@@ -431,17 +432,17 @@ static BOOL CheckHasOids(StatementClass * stmt)
 		res = NULL;
 		if (!hasoids && !hassubclass)
 		{
-			sprintf(query, "select a.attname, a.atttypid from pg_index i, pg_attribute a where indrelid=%u and indnatts=1 and indisunique and indexprs is null and indpred is null and i.indrelid = a.attrelid and a.attnum=i.indkey[0] and attnotnull and atttypid in (%d, %d)", ti->table_oid, PG_TYPE_INT4, PG_TYPE_OID);
-			res = CC_send_query(conn, query, NULL, ROLLBACK_ON_ERROR | IGNORE_ABORT_ON_CONN, NULL);
+			SPRINTF_FIXED(query, "select a.attname, a.atttypid from pg_index i, pg_attribute a where indrelid=%u and indnatts=1 and indisunique and indexprs is null and indpred is null and i.indrelid = a.attrelid and a.attnum=i.indkey[0] and attnotnull and atttypid in (%d, %d)", ti->table_oid, PG_TYPE_INT4, PG_TYPE_OID);
+			res = CC_send_query(conn, query, NULL, READ_ONLY_QUERY, NULL);
 			if (QR_command_maybe_successful(res) && QR_get_num_total_tuples(res) > 0)
 			{
 				foundKey = TRUE;
 				STR_TO_NAME(ti->bestitem, QR_get_value_backend_text(res, 0, 0));
-				sprintf(query, "\"%s\" = %%", SAFE_NAME(ti->bestitem));
+				SPRINTF_FIXED(query, "\"%s\" = %%", SAFE_NAME(ti->bestitem));
 				if (PG_TYPE_INT4 == (OID) QR_get_value_backend_int(res, 0, 1, NULL))
-					strcat(query, "d");
+					STRCAT_FIXED(query, "d");
 				else
-					strcat(query, "u");
+					STRCAT_FIXED(query, "u");
 				STRX_TO_NAME(ti->bestqual, query);
 			}
 			else
@@ -767,11 +768,11 @@ COL_INFO **coli)
 			/*
 			 * We also have to check as follows.
 			 */
-			snprintf(token, sizeof(token),
+			SPRINTF_FIXED(token,
 					 "select nspname from pg_namespace n, pg_class c"
 					 " where c.relnamespace=n.oid and c.oid='\"%s\"'::regclass",
 					 SAFE_NAME(table_name));
-			res = CC_send_query(conn, token, NULL, ROLLBACK_ON_ERROR | IGNORE_ABORT_ON_CONN, NULL);
+			res = CC_send_query(conn, token, NULL, READ_ONLY_QUERY, NULL);
 			if (QR_command_maybe_successful(res))
 			{
 				if (QR_get_num_total_tuples(res) == 1)
@@ -1305,7 +1306,7 @@ parse_the_statement(StatementClass *stmt, BOOL check_hasoids, BOOL sqlsvr_check)
 
 	delim = '\0';
 	token[0] = '\0';
-	while (pptr = ptr, (delim != ',') ? strcpy(btoken, token) : (btoken[0] = '\0', NULL), (ptr = getNextToken(conn->ccsc, CC_get_escape(conn), pptr, token, sizeof(token), &delim, &quote, &dquote, &numeric)) != NULL)
+	while (pptr = ptr, (delim != ',') ? STRCPY_FIXED(btoken, token) : (btoken[0] = '\0', NULL), (ptr = getNextToken(conn->ccsc, CC_get_escape(conn), pptr, token, sizeof(token), &delim, &quote, &dquote, &numeric)) != NULL)
 	{
 		unquoted = !(quote || dquote);
 
@@ -1426,7 +1427,7 @@ parse_the_statement(StatementClass *stmt, BOOL check_hasoids, BOOL sqlsvr_check)
 					subqlevel = 0;
 			}
 			if (blevel >= old_blevel && ',' != delim)
-				strcpy(stoken, token);
+				STRCPY_FIXED(stoken, token);
 			else
 				stoken[0] = '\0';
 		}
