@@ -27,6 +27,26 @@
 #define	EXPERIMENTAL_CURRENTLY
 
 
+SQLSMALLINT ansi_to_wtype(const ConnectionClass *self, SQLSMALLINT ansitype)
+{
+#ifndef	UNICODE_SUPPORT
+	return ansitype;
+#else
+	if (!ALLOW_WCHAR(self))
+		return ansitype;
+	switch (ansitype)
+	{
+		case SQL_CHAR:
+			return SQL_WCHAR;
+		case SQL_VARCHAR:
+			return SQL_WVARCHAR;
+		case SQL_LONGVARCHAR:
+			return SQL_WLONGVARCHAR;
+	}
+	return ansitype;
+#endif /* UNICODE_SUPPORT */
+}
+
 Int4		getCharColumnSize(const StatementClass *stmt, OID type, int col, int handle_unknown_size_as);
 
 /*
@@ -287,37 +307,31 @@ inolog("!!! atttypmod  < 0 ?\n");
 	if (atttypmod < 0 && adtsize_or_longestlen < 0)
 		return maxsize;
 
+inolog("!!! adtsize_or_logngest=%d\n", adtsize_or_longestlen);
+	p = adtsize_or_longestlen; /* longest */
 	/*
 	 * Catalog Result Sets -- use assigned column width (i.e., from
 	 * set_tuplefield_string)
 	 */
 inolog("!!! catalog_result=%d\n", handle_unknown_size_as);
-	if (UNKNOWNS_AS_CATALOG == handle_unknown_size_as)
+	if (UNKNOWNS_AS_LONGEST == handle_unknown_size_as)
 	{
-		if (adtsize_or_longestlen > 0)
-			return adtsize_or_longestlen;
-		return maxsize;
+		mylog("%s: LONGEST: p = %d\n", __FUNCTION__, p);
+		if (p > 0 &&
+		    (atttypmod < 0 || atttypmod > p))
+			return p;
 	}
 	if (TYPE_MAY_BE_ARRAY(type))
 	{
-		if (adtsize_or_longestlen > 0)
-			return adtsize_or_longestlen;
+		if (p > 0)
+			return p;
 		return maxsize;
 	}
 
-inolog("!!! adtsize_or_logngest=%d\n", adtsize_or_longestlen);
-	p = adtsize_or_longestlen; /* longest */
 	/* Size is unknown -- handle according to parameter */
 	if (atttypmod > 0)	/* maybe the length is known */
 	{
-		if (atttypmod >= p)
-			return atttypmod;
-		switch (type)
-		{
-			case PG_TYPE_VARCHAR:
-			case PG_TYPE_BPCHAR:
-				return atttypmod;
-		}
+		return atttypmod;
 	}
 
 	/* The type is really unknown */
@@ -326,10 +340,6 @@ inolog("!!! adtsize_or_logngest=%d\n", adtsize_or_longestlen);
 		case UNKNOWNS_AS_DONTKNOW:
 			return -1;
 		case UNKNOWNS_AS_LONGEST:
-			mylog("%s: LONGEST: p = %d\n", __FUNCTION__, p);
-			if (p > 0)
-				return p;
-			break;
 		case UNKNOWNS_AS_MAX:
 			break;
 		default:
@@ -350,8 +360,13 @@ inolog("!!! adtsize_or_logngest=%d\n", adtsize_or_longestlen);
 	return maxsize;
 }
 
+/*
+ *	Specify when handle_unknown_size_as parameter is unused 
+ */
+#define	UNUSED_HANDLE_UNKNOWN_SIZE_AS	(-2)
+
 static SQLSMALLINT
-getNumericDecimalDigitsX(const ConnectionClass *conn, OID type, int atttypmod, int adtsize_or_longest, int handle_unknown_size_as)
+getNumericDecimalDigitsX(const ConnectionClass *conn, OID type, int atttypmod, int adtsize_or_longest, int UNUSED_handle_unknown_size_as)
 {
 	Int4		default_decimal_digits = 6;
 
@@ -389,8 +404,6 @@ getNumericColumnSizeX(const ConnectionClass *conn, OID type, int atttypmod, int
 	{
 		case UNKNOWNS_AS_MAX:
 			return adtsize_or_longest > default_column_size ? adtsize_or_longest : default_column_size;
-		case UNKNOWNS_AS_CATALOG:
-			break;
 		default:
 			if (adtsize_or_longest < 10)
 				adtsize_or_longest = 10;
@@ -492,52 +505,41 @@ getIntervalColumnSize(OID type, int atttypmod)
 
 
 SQLSMALLINT
-pgtype_attr_to_concise_type(const ConnectionClass *conn, OID type, int atttypmod, int adtsize_or_longestlen)
+pgtype_attr_to_concise_type(const ConnectionClass *conn, OID type, int atttypmod, int adtsize_or_longestlen, int handle_unknown_size_as)
 {
 	const ConnInfo	*ci = &(conn->connInfo);
 	EnvironmentClass *env = (EnvironmentClass *) CC_get_env(conn);
 #ifdef	PG_INTERVAL_AS_SQL_INTERVAL
 	SQLSMALLINT	sqltype;
 #endif /* PG_INTERVAL_AS_SQL_INTERVAL */
+	BOOL	bLongVarchar, bFixed = FALSE;
 
 	switch (type)
 	{
 		case PG_TYPE_CHAR:
-			return ALLOW_WCHAR(conn) ? SQL_WCHAR : SQL_CHAR;
+			return ansi_to_wtype(conn, SQL_CHAR);
 		case PG_TYPE_NAME:
 		case PG_TYPE_REFCURSOR:
-			return ALLOW_WCHAR(conn) ? SQL_WVARCHAR : SQL_VARCHAR;
+			return ansi_to_wtype(conn, SQL_VARCHAR);
 
-#ifdef	UNICODE_SUPPORT
 		case PG_TYPE_BPCHAR:
-			if (getCharColumnSizeX(conn, type, atttypmod, adtsize_or_longestlen, UNKNOWNS_AS_DEFAULT) > ci->drivers.max_varchar_size)
-				return ALLOW_WCHAR(conn) ? SQL_WLONGVARCHAR : SQL_LONGVARCHAR;
-			return ALLOW_WCHAR(conn) ? SQL_WCHAR : SQL_CHAR;
-
+			bFixed = TRUE;
 		case PG_TYPE_VARCHAR:
-			if (getCharColumnSizeX(conn, type, atttypmod, adtsize_or_longestlen, UNKNOWNS_AS_DEFAULT) > ci->drivers.max_varchar_size)
-				return ALLOW_WCHAR(conn) ? SQL_WLONGVARCHAR : SQL_LONGVARCHAR;
-			return ALLOW_WCHAR(conn) ? SQL_WVARCHAR : SQL_VARCHAR;
-
+			if (getCharColumnSizeX(conn, type, atttypmod, adtsize_or_longestlen, handle_unknown_size_as) > ci->drivers.max_varchar_size)
+				bLongVarchar = TRUE;
+			else
+				bLongVarchar = FALSE;
+			return ansi_to_wtype(conn, bLongVarchar ? SQL_LONGVARCHAR : (bFixed ? SQL_CHAR : SQL_VARCHAR));
 		case PG_TYPE_TEXT:
-			return ci->drivers.text_as_longvarchar ?
-				(ALLOW_WCHAR(conn) ? SQL_WLONGVARCHAR : SQL_LONGVARCHAR) :
-				(ALLOW_WCHAR(conn) ? SQL_WVARCHAR : SQL_VARCHAR);
-
-#else
-		case PG_TYPE_BPCHAR:
-			if (getCharColumnSizeX(conn, type, atttypmod, adtsize_or_longestlen, UNKNOWNS_AS_DEFAULT) > ci->drivers.max_varchar_size)
-				return SQL_LONGVARCHAR;
-			return SQL_CHAR;
-
-		case PG_TYPE_VARCHAR:
-			if (getCharColumnSizeX(conn, type, atttypmod, adtsize_or_longestlen, UNKNOWNS_AS_DEFAULT) > ci->drivers.max_varchar_size)
-				return SQL_LONGVARCHAR;
-			return SQL_VARCHAR;
-
-		case PG_TYPE_TEXT:
-			return ci->drivers.text_as_longvarchar ? SQL_LONGVARCHAR : SQL_VARCHAR;
-#endif /* UNICODE_SUPPORT */
+			bLongVarchar = ci->drivers.text_as_longvarchar;
+			if (bLongVarchar)
+			{
+				int column_size = getCharColumnSizeX(conn, type, atttypmod, adtsize_or_longestlen, handle_unknown_size_as);
+				if (column_size > 0 &&
+				    column_size <= ci->drivers.max_varchar_size)
+					bLongVarchar = FALSE;
+			}
+			return ansi_to_wtype(conn, bLongVarchar ? SQL_LONGVARCHAR : SQL_VARCHAR);
 
 		case PG_TYPE_BYTEA:
 			if (ci->bytea_as_longvarbinary)
@@ -590,21 +592,20 @@ pgtype_attr_to_concise_type(const ConnectionClass *conn, OID type, int atttypmod
 		case PG_TYPE_BOOL:
 			return ci->drivers.bools_as_char ? SQL_VARCHAR : SQL_BIT;
 		case PG_TYPE_XML:
-			return ALLOW_WCHAR(conn) ? SQL_WLONGVARCHAR : SQL_LONGVARCHAR;
+			return ansi_to_wtype(conn, SQL_LONGVARCHAR);
 		case PG_TYPE_INET:
 		case PG_TYPE_CIDR:
 		case PG_TYPE_MACADDR:
-			return ALLOW_WCHAR(conn) ? SQL_WVARCHAR : SQL_VARCHAR;
+			return ansi_to_wtype(conn, SQL_VARCHAR);
 		case PG_TYPE_UUID:
 			return SQL_GUID;
-			return ALLOW_WCHAR(conn) ? SQL_WVARCHAR : SQL_VARCHAR;
 
 		case PG_TYPE_INTERVAL:
 #ifdef	PG_INTERVAL_AS_SQL_INTERVAL
 			if (sqltype = get_interval_type(atttypmod, NULL), 0 != sqltype)
 				return sqltype;
 #endif /* PG_INTERVAL_AS_SQL_INTERVAL */
-			return CC_is_in_unicode_driver(conn) ? SQL_WVARCHAR : SQL_VARCHAR;
+			return ansi_to_wtype(conn, SQL_VARCHAR);
 
 		default:
 
@@ -617,16 +618,23 @@ pgtype_attr_to_concise_type(const ConnectionClass *conn, OID type, int atttypmod
 			if (type == conn->lobj_type)
 				return SQL_LONGVARBINARY;
 
+			bLongVarchar = ci->drivers.unknowns_as_longvarchar;
+			if (bLongVarchar)
+			{
+				int column_size = getCharColumnSizeX(conn, type, atttypmod, adtsize_or_longestlen, handle_unknown_size_as);
+				if (column_size > 0 &&
+				    column_size <= ci->drivers.max_varchar_size)
+					bLongVarchar = FALSE;
+			}
 #ifdef	EXPERIMENTAL_CURRENTLY
-			if (ALLOW_WCHAR(conn))
-				return ci->drivers.unknowns_as_longvarchar ? SQL_WLONGVARCHAR : SQL_WVARCHAR;
+			return ansi_to_wtype(conn, bLongVarchar ? SQL_LONGVARCHAR : SQL_VARCHAR);
 #endif	/* EXPERIMENTAL_CURRENTLY */
-			return ci->drivers.unknowns_as_longvarchar ? SQL_LONGVARCHAR : SQL_VARCHAR;
+			return bLongVarchar ? SQL_LONGVARCHAR : SQL_VARCHAR;
 	}
 }
 
 SQLSMALLINT
-pgtype_attr_to_sqldesctype(const ConnectionClass *conn, OID type, int atttypmod)
+pgtype_attr_to_sqldesctype(const ConnectionClass *conn, OID type, int atttypmod, int adtsize_or_longestlen, int handle_unknown_size_as)
 {
 	SQLSMALLINT	rettype;
 
@@ -634,7 +642,7 @@ pgtype_attr_to_sqldesctype(const ConnectionClass *conn, OID type, int atttypmod)
 	if (PG_TYPE_INTERVAL == type)
 		return SQL_INTERVAL;
 #endif /* PG_INTERVAL_AS_SQL_INTERVAL */
-	switch (rettype = pgtype_attr_to_concise_type(conn, type, atttypmod, PG_UNSPECIFIED))
+	switch (rettype = pgtype_attr_to_concise_type(conn, type, atttypmod, adtsize_or_longestlen, handle_unknown_size_as))
 	{
 		case SQL_TYPE_DATE:
 		case SQL_TYPE_TIME:
@@ -649,7 +657,7 @@ pgtype_attr_to_datetime_sub(const ConnectionClass *conn, OID type, int atttypmod
 {
 	SQLSMALLINT	rettype;
 
-	switch (rettype = pgtype_attr_to_concise_type(conn, type, atttypmod, PG_UNSPECIFIED))
+	switch (rettype = pgtype_attr_to_concise_type(conn, type, atttypmod, PG_ADT_UNSET, PG_UNKNOWNS_UNSET))
 	{
 		case SQL_TYPE_DATE:
 			return SQL_CODE_DATE;
@@ -727,23 +735,21 @@ pgtype_attr_to_ctype(const ConnectionClass *conn, OID type, int atttypmod)
 			return SQL_C_BINARY;
 		case PG_TYPE_LO_UNDEFINED:
 			return SQL_C_BINARY;
-#ifdef	UNICODE_SUPPORT
 		case PG_TYPE_BPCHAR:
 		case PG_TYPE_VARCHAR:
 		case PG_TYPE_TEXT:
-			return ALLOW_WCHAR(conn) ? SQL_C_WCHAR : SQL_C_CHAR;
-#endif /* UNICODE_SUPPORT */
+			return ansi_to_wtype(conn, SQL_C_CHAR);
 		case PG_TYPE_UUID:
 			if (!conn->ms_jet)
 				return SQL_C_GUID;
-			return ALLOW_WCHAR(conn) ? SQL_C_WCHAR : SQL_C_CHAR;
+			return ansi_to_wtype(conn, SQL_C_CHAR);
 
 		case PG_TYPE_INTERVAL:
 #ifdef	PG_INTERVAL_AS_SQL_INTERVAL
 			if (ctype = get_interval_type(atttypmod, NULL), 0 != ctype)
 				return ctype;
 #endif /* PG_INTERVAL_AS_SQL_INTERVAL */
-			return CC_is_in_unicode_driver(conn) ? SQL_C_WCHAR : SQL_CHAR;
+			return ansi_to_wtype(conn, SQL_CHAR);
 
 		default:
 			/* hack until permanent type is available */
@@ -752,8 +758,7 @@ pgtype_attr_to_ctype(const ConnectionClass *conn, OID type, int atttypmod)
 
 			/* Experimental, Does this work ? */
 #ifdef	EXPERIMENTAL_CURRENTLY
-			if (ALLOW_WCHAR(conn))
-				return SQL_C_WCHAR;
+			return ansi_to_wtype(conn, SQL_C_CHAR);
 #endif	/* EXPERIMENTAL_CURRENTLY */
 			return SQL_C_CHAR;
 	}
@@ -848,8 +853,6 @@ pgtype_attr_column_size(const ConnectionClass *conn, OID type, int atttypmod, in
 {
 	const ConnInfo	*ci = &(conn->connInfo);
 
-	if (handle_unknown_size_as == UNKNOWNS_AS_DEFAULT)
-		handle_unknown_size_as = ci->drivers.unknown_sizes;
 	switch (type)
 	{
 		case PG_TYPE_CHAR:
@@ -1144,7 +1147,7 @@ pgtype_attr_desclength(const ConnectionClass *conn, OID type, int atttypmod, int
 }
 
 Int2
-pgtype_attr_decimal_digits(const ConnectionClass *conn, OID type, int atttypmod, int adtsize_or_longestlen, int handle_unknown_size_as)
+pgtype_attr_decimal_digits(const ConnectionClass *conn, OID type, int atttypmod, int adtsize_or_longestlen, int UNUSED_handle_unknown_size_as)
 {
 	switch (type)
 	{
@@ -1172,7 +1175,7 @@ pgtype_attr_decimal_digits(const ConnectionClass *conn, OID type, int atttypmod,
 			return getTimestampDecimalDigitsX(conn, type, atttypmod);
 
 		case PG_TYPE_NUMERIC:
-			return getNumericDecimalDigitsX(conn, type, atttypmod, adtsize_or_longestlen, handle_unknown_size_as);
+			return getNumericDecimalDigitsX(conn, type, atttypmod, adtsize_or_longestlen, UNUSED_handle_unknown_size_as);
 
 #ifdef	PG_INTERVAL_AS_SQL_INTERVAL
 		case PG_TYPE_INTERVAL:
@@ -1185,12 +1188,12 @@ pgtype_attr_decimal_digits(const ConnectionClass *conn, OID type, int atttypmod,
 }
 
 Int2
-pgtype_attr_scale(const ConnectionClass *conn, OID type, int atttypmod, int adtsize_or_longestlen, int handle_unknown_size_as)
+pgtype_attr_scale(const ConnectionClass *conn, OID type, int atttypmod, int adtsize_or_longestlen, int UNUSED_handle_unknown_size_as)
 {
 	switch (type)
 	{
 		case PG_TYPE_NUMERIC:
-			return getNumericDecimalDigitsX(conn, type, atttypmod, adtsize_or_longestlen, handle_unknown_size_as);
+			return getNumericDecimalDigitsX(conn, type, atttypmod, adtsize_or_longestlen, UNUSED_handle_unknown_size_as);
 	}
 	return -1;
 }
@@ -1207,7 +1210,7 @@ pgtype_attr_transfer_octet_length(const ConnectionClass *conn, OID type, int att
 		case PG_TYPE_BPCHAR:
 		case PG_TYPE_TEXT:
 		case PG_TYPE_UNKNOWN:
-			column_size = pgtype_attr_column_size(conn, type, atttypmod, PG_UNSPECIFIED, handle_unknown_size_as);
+			column_size = pgtype_attr_column_size(conn, type, atttypmod, PG_ADT_UNSET, handle_unknown_size_as);
 			if (SQL_NO_TOTAL == column_size)
 				return column_size;
 #ifdef	UNICODE_SUPPORT
@@ -1225,10 +1228,10 @@ pgtype_attr_transfer_octet_length(const ConnectionClass *conn, OID type, int att
 				return maxvarc;
 			return coef * column_size;
 		case PG_TYPE_BYTEA:
-			return pgtype_attr_column_size(conn, type, atttypmod, PG_UNSPECIFIED, handle_unknown_size_as);
+			return pgtype_attr_column_size(conn, type, atttypmod, PG_ADT_UNSET, handle_unknown_size_as);
 		default:
 			if (type == conn->lobj_type)
-				return pgtype_attr_column_size(conn, type, atttypmod, PG_UNSPECIFIED, handle_unknown_size_as);
+				return pgtype_attr_column_size(conn, type, atttypmod, PG_ADT_UNSET, handle_unknown_size_as);
 	}
 	return -1;
 }
@@ -1416,7 +1419,7 @@ sqltype_to_pgtype(const ConnectionClass *conn, SQLSMALLINT fSqlType)
 			pgType = PG_TYPE_VARCHAR;
 			break;
 
-#if	UNICODE_SUPPORT
+#ifdef	UNICODE_SUPPORT
 		case SQL_WVARCHAR:
 			pgType = PG_TYPE_VARCHAR;
 			break;
@@ -1453,7 +1456,7 @@ getAtttypmodEtc(const StatementClass *stmt, int col, int *adtsize_or_longestlen)
 	int	atttypmod = -1;
 
 	if (NULL != adtsize_or_longestlen)
-		*adtsize_or_longestlen = -1;
+		*adtsize_or_longestlen = PG_ADT_UNSET;
 	if (col >= 0)
 	{
 		const QResultClass	*res;
@@ -1512,20 +1515,21 @@ getAtttypmodEtc(const StatementClass *stmt, int col, int *adtsize_or_longestlen)
  *	types that are unknown.  All other pg routines in here return a suitable default.
  */
 SQLSMALLINT
-pgtype_to_concise_type(const StatementClass *stmt, OID type, int col)
+pgtype_to_concise_type(const StatementClass *stmt, OID type, int col, int handle_unknown_size_as)
 {
 	int	atttypmod, adtsize_or_longestlen;
 
 	atttypmod = getAtttypmodEtc(stmt, col, &adtsize_or_longestlen);
-	return pgtype_attr_to_concise_type(SC_get_conn(stmt), type, atttypmod, adtsize_or_longestlen);
+	return pgtype_attr_to_concise_type(SC_get_conn(stmt), type, atttypmod, adtsize_or_longestlen, handle_unknown_size_as);
 }
 
 SQLSMALLINT
-pgtype_to_sqldesctype(const StatementClass *stmt, OID type, int col)
+pgtype_to_sqldesctype(const StatementClass *stmt, OID type, int col, int handle_unknown_size_as)
 {
-	int	atttypmod = getAtttypmodEtc(stmt, col, NULL);
+	int	adtsize_or_longestlen;
+	int	atttypmod = getAtttypmodEtc(stmt, col, &adtsize_or_longestlen);
 
-	return pgtype_attr_to_sqldesctype(SC_get_conn(stmt), type, atttypmod);
+	return pgtype_attr_to_sqldesctype(SC_get_conn(stmt), type, atttypmod, adtsize_or_longestlen, handle_unknown_size_as);
 }
 
 SQLSMALLINT
@@ -1545,258 +1549,6 @@ pgtype_to_name(const StatementClass *stmt, OID type, int col, BOOL auto_incremen
 }
 
 
-#ifdef	NOT_USED
-static SQLSMALLINT
-getNumericDecimalDigits(const StatementClass *stmt, OID type, int col)
-{
-	Int4		atttypmod = -1, default_decimal_digits = 6;
-	QResultClass	*result;
-	ColumnInfoClass *flds;
-
-	mylog("getNumericDecimalDigits: type=%d, col=%d\n", type, col);
-
-	if (col < 0)
-		return default_decimal_digits;
-
-	result = SC_get_Curres(stmt);
-
-	/*
-	 * Manual Result Sets -- use assigned column width (i.e., from
-	 * set_tuplefield_string)
-	 */
-	atttypmod = QR_get_atttypmod(result, col);
-	if (atttypmod > -1)
-		return (atttypmod & 0xffff);
-	if (stmt->catalog_result)
-	{
-		flds = QR_get_fields(result);
-		if (flds)
-		{
-			int	fsize = CI_get_fieldsize(flds, col);
-			if (fsize > 0)
-				return fsize;
-		}
-		return default_decimal_digits;
-	}
-	else
-	{
-		Int4 dsp_size = QR_get_display_size(result, col);
-		if (dsp_size <= 0)
-			return default_decimal_digits;
-		if (dsp_size < 5)
-			dsp_size = 5;
-		return dsp_size;
-	}
-}
-
-
-static Int4	/* PostgreSQL restritiction */
-getNumericColumnSize(const StatementClass *stmt, OID type, int col)
-{
-	Int4	atttypmod = -1, default_column_size = 28;
-	QResultClass *result;
-	ColumnInfoClass *flds;
-
-	mylog("getNumericColumnSize: type=%d, col=%d\n", type, col);
-
-	if (col < 0)
-		return default_column_size;
-
-	result = SC_get_Curres(stmt);
-
-	/*
-	 * Manual Result Sets -- use assigned column width (i.e., from
-	 * set_tuplefield_string)
-	 */
-	atttypmod = QR_get_atttypmod(result, col);
-	if (atttypmod > -1)
-		return (atttypmod >> 16) & 0xffff;
-	if (stmt->catalog_result)
-	{
-		flds = QR_get_fields(result);
-		if (flds)
-		{
-			int	fsize = CI_get_fieldsize(flds, col);
-			if (fsize > 0)
-				return 2 * fsize;
-		}
-		return default_column_size;
-	}
-	else
-	{
-		Int4	dsp_size = QR_get_display_size(result, col);
-		if (dsp_size <= 0)
-			return default_column_size;
-		dsp_size *= 2;
-		if (dsp_size < 10)
-			dsp_size = 10;
-		return dsp_size;
-	}
-}
-
-#ifdef NOT_USED
-Int4
-getCharColumnSize(const StatementClass *stmt, OID type, int col, int handle_unknown_size_as)
-{
-	CSTR	func = "getCharColumnSize";
-	int		p = -1, attlen = -1, adtsize = -1, maxsize;
-	QResultClass	*result;
-	ConnectionClass	*conn = SC_get_conn(stmt);
-	ConnInfo	*ci = &(conn->connInfo);
-
-	mylog("%s: type=%d, col=%d, unknown = %d\n", func, type, col, handle_unknown_size_as);
-
-	/* Assign Maximum size based on parameters */
-	switch (type)
-	{
-		case PG_TYPE_TEXT:
-			if (ci->drivers.text_as_longvarchar)
-				maxsize = ci->drivers.max_longvarchar_size;
-			else
-				maxsize = ci->drivers.max_varchar_size;
-			break;
-
-		case PG_TYPE_VARCHAR:
-		case PG_TYPE_BPCHAR:
-			maxsize = ci->drivers.max_varchar_size;
-			break;
-
-		default:
-			if (ci->drivers.unknowns_as_longvarchar)
-				maxsize = ci->drivers.max_longvarchar_size;
-			else
-				maxsize = ci->drivers.max_varchar_size;
-			break;
-	}
-#ifdef	UNICODE_SUPPORT
-	if (CC_is_in_unicode_driver(conn) &&
-	    isSqlServr() &&
-	    maxsize > 4000)
-		maxsize = 4000;
-#endif /* UNICODE_SUPPORT */
-
-	if (maxsize == TEXT_FIELD_SIZE + 1) /* magic length for testing */
-		maxsize = 0;
-
-	/*
-	 * Static ColumnSize (i.e., the Maximum ColumnSize of the datatype) This
-	 * has nothing to do with a result set.
-	 */
-	if (col < 0)
-		return maxsize;
-
-	if (result = SC_get_Curres(stmt), NULL == result)
-		return maxsize;
-
-	/*
-	 * Catalog Result Sets -- use assigned column width (i.e., from
-	 * set_tuplefield_string)
-	 */
-	adtsize = QR_get_fieldsize(result, col);
-	if (stmt->catalog_result)
-	{
-		if (adtsize > 0)
-			return adtsize;
-		return maxsize;
-	}
-
-	p = QR_get_display_size(result, col); /* longest */
-	attlen = QR_get_atttypmod(result, col);
-	/* Size is unknown -- handle according to parameter */
-	if (attlen > 0)	/* maybe the length is known */
-	{
-		if (attlen >= p)
-			return attlen;
-		switch (type)
-		{
-			case PG_TYPE_VARCHAR:
-			case PG_TYPE_BPCHAR:
-				return attlen;
-		}
-	}
-
-	/* The type is really unknown */
-	switch (handle_unknown_size_as)
-	{
-		case UNKNOWNS_AS_DONTKNOW:
-			return -1;
-		case UNKNOWNS_AS_LONGEST:
-			mylog("%s: LONGEST: p = %d\n", func, p);
-			if (p > 0)
-				return p;
-			break;
-		case UNKNOWNS_AS_MAX:
-			break;
-		default:
-			return -1;
-	}
-	if (maxsize <= 0)
-		return maxsize;
-	switch (type)
-	{
-		case PG_TYPE_BPCHAR:
-		case PG_TYPE_VARCHAR:
-		case PG_TYPE_TEXT:
-			return maxsize;
-	}
-
-	if (p > maxsize)
-		maxsize = p;
-	return maxsize;
-}
-#endif /* NOT_USED */
-
-static SQLSMALLINT
-getTimestampDecimalDigits(const StatementClass *stmt, OID type, int col)
-{
-	ConnectionClass *conn = SC_get_conn(stmt);
-	Int4		atttypmod;
-	QResultClass *result;
-
-	mylog("getTimestampDecimalDigits: type=%d, col=%d\n", type, col);
-
-	if (col < 0)
-		return 0;
-
-	result = SC_get_Curres(stmt);
-
-	atttypmod = QR_get_atttypmod(result, col);
-	mylog("atttypmod2=%d\n", atttypmod);
-	return (atttypmod > -1 ? atttypmod : 6);
-}
-
-
-static SQLSMALLINT
-getTimestampColumnSize(const StatementClass *stmt, OID type, int col)
-{
-	Int4		fixed,
-				scale;
-
-	mylog("getTimestampColumnSize: type=%d, col=%d\n", type, col);
-
-	switch (type)
-	{
-		case PG_TYPE_TIME:
-			fixed = 8;
-			break;
-		case PG_TYPE_TIME_WITH_TMZONE:
-			fixed = 11;
-			break;
-		case PG_TYPE_TIMESTAMP_NO_TMZONE:
-			fixed = 19;
-			break;
-		default:
-			if (USE_ZONE)
-				fixed = 22;
-			else
-				fixed = 19;
-			break;
-	}
-	scale = getTimestampDecimalDigits(stmt, type, col);
-	return (scale > 0) ? fixed + 1 + scale : fixed;
-}
-#endif /* NOT_USED */
-
 /*
  *	This corresponds to "precision" in ODBC 2.x.
  *
@@ -1812,7 +1564,7 @@ pgtype_column_size(const StatementClass *stmt, OID type, int col, int handle_unk
 	int	atttypmod, adtsize_or_longestlen;
 
 	atttypmod = getAtttypmodEtc(stmt, col, &adtsize_or_longestlen);
-	return pgtype_attr_column_size(SC_get_conn(stmt), type, atttypmod, adtsize_or_longestlen, stmt->catalog_result ? UNKNOWNS_AS_CATALOG : handle_unknown_size_as);
+	return pgtype_attr_column_size(SC_get_conn(stmt), type, atttypmod, adtsize_or_longestlen, stmt->catalog_result ? UNKNOWNS_AS_LONGEST : handle_unknown_size_as);
 }
 
 /*
@@ -1824,7 +1576,7 @@ pgtype_precision(const StatementClass *stmt, OID type, int col, int handle_unkno
 	int	atttypmod, adtsize_or_longestlen;
 
 	atttypmod = getAtttypmodEtc(stmt, col, &adtsize_or_longestlen);
-	return pgtype_attr_precision(SC_get_conn(stmt), type, atttypmod, adtsize_or_longestlen, stmt->catalog_result ? UNKNOWNS_AS_CATALOG : handle_unknown_size_as);
+	return pgtype_attr_precision(SC_get_conn(stmt), type, atttypmod, adtsize_or_longestlen, stmt->catalog_result ? UNKNOWNS_AS_LONGEST : handle_unknown_size_as);
 }
 
 
@@ -1834,7 +1586,7 @@ pgtype_display_size(const StatementClass *stmt, OID type, int col, int handle_un
 	int	atttypmod, adtsize_or_longestlen;
 
 	atttypmod = getAtttypmodEtc(stmt, col, &adtsize_or_longestlen);
-	return pgtype_attr_display_size(SC_get_conn(stmt), type, atttypmod, adtsize_or_longestlen, stmt->catalog_result ? UNKNOWNS_AS_CATALOG : handle_unknown_size_as);
+	return pgtype_attr_display_size(SC_get_conn(stmt), type, atttypmod, adtsize_or_longestlen, stmt->catalog_result ? UNKNOWNS_AS_LONGEST : handle_unknown_size_as);
 }
 
 
@@ -1848,7 +1600,7 @@ pgtype_buffer_length(const StatementClass *stmt, OID type, int col, int handle_u
 	int	atttypmod, adtsize_or_longestlen;
 
 	atttypmod = getAtttypmodEtc(stmt, col, &adtsize_or_longestlen);
-	return pgtype_attr_buffer_length(SC_get_conn(stmt), type, atttypmod, adtsize_or_longestlen, stmt->catalog_result ? UNKNOWNS_AS_CATALOG : handle_unknown_size_as);
+	return pgtype_attr_buffer_length(SC_get_conn(stmt), type, atttypmod, adtsize_or_longestlen, stmt->catalog_result ? UNKNOWNS_AS_LONGEST : handle_unknown_size_as);
 }
 
 /*
@@ -1859,7 +1611,7 @@ pgtype_desclength(const StatementClass *stmt, OID type, int col, int handle_unkn
 	int	atttypmod, adtsize_or_longestlen;
 
 	atttypmod = getAtttypmodEtc(stmt, col, &adtsize_or_longestlen);
-	return pgtype_attr_desclength(SC_get_conn(stmt), type, atttypmod, adtsize_or_longestlen, stmt->catalog_result ? UNKNOWNS_AS_CATALOG : handle_unknown_size_as);
+	return pgtype_attr_desclength(SC_get_conn(stmt), type, atttypmod, adtsize_or_longestlen, stmt->catalog_result ? UNKNOWNS_AS_LONGEST : handle_unknown_size_as);
 }
 
 #ifdef	NOT_USED
@@ -1956,7 +1708,7 @@ pgtype_max_decimal_digits(const ConnectionClass *conn, OID type)
 		case PG_TYPE_TIMESTAMP_NO_TMZONE:
 			return 38;
 		case PG_TYPE_NUMERIC:
-			return getNumericDecimalDigitsX(conn, type, -1, -1, UNKNOWNS_AS_DEFAULT);
+			return getNumericDecimalDigitsX(conn, type, -1, -1, UNUSED_HANDLE_UNKNOWN_SIZE_AS);
 		default:
 			return -1;
 	}
@@ -1971,7 +1723,7 @@ pgtype_decimal_digits(const StatementClass *stmt, OID type, int col)
 	int	atttypmod, adtsize_or_longestlen;
 
 	atttypmod = getAtttypmodEtc(stmt, col, &adtsize_or_longestlen);
-	return pgtype_attr_decimal_digits(SC_get_conn(stmt), type, atttypmod, adtsize_or_longestlen, stmt->catalog_result ? UNKNOWNS_AS_CATALOG : UNKNOWNS_AS_DEFAULT);
+	return pgtype_attr_decimal_digits(SC_get_conn(stmt), type, atttypmod, adtsize_or_longestlen, UNUSED_HANDLE_UNKNOWN_SIZE_AS);
 }
 
 /*
@@ -1983,7 +1735,7 @@ pgtype_scale(const StatementClass *stmt, OID type, int col)
 	int	atttypmod, adtsize_or_longestlen;
 
 	atttypmod = getAtttypmodEtc(stmt, col, &adtsize_or_longestlen);
-	return pgtype_attr_scale(SC_get_conn(stmt), type, atttypmod, adtsize_or_longestlen, stmt->catalog_result ? UNKNOWNS_AS_CATALOG : UNKNOWNS_AS_DEFAULT);
+	return pgtype_attr_scale(SC_get_conn(stmt), type, atttypmod, adtsize_or_longestlen, UNUSED_HANDLE_UNKNOWN_SIZE_AS);
 }
 
 
@@ -2207,9 +1959,7 @@ sqltype_to_default_ctype(const ConnectionClass *conn, SQLSMALLINT sqltype)
 		case SQL_WCHAR:
 		case SQL_WVARCHAR:
 		case SQL_WLONGVARCHAR:
-			if (!ALLOW_WCHAR(conn))
-				return SQL_C_CHAR;
-			return SQL_C_WCHAR;
+			return ansi_to_wtype(conn, SQL_C_CHAR);
 #endif /* UNICODE_SUPPORT */
 
 		case SQL_BIT:
