@@ -384,7 +384,7 @@ SC_Constructor(ConnectionClass *conn)
 		rv->prepare = NON_PREPARE_STATEMENT;
 		rv->prepared = NOT_YET_PREPARED;
 		rv->status = STMT_ALLOCATED;
-		rv->internal = FALSE;
+		rv->external = FALSE;
 		rv->iflag = 0;
 		rv->plan_name = NULL;
 		rv->transition_status = STMT_TRANSITION_UNALLOCATED;
@@ -417,7 +417,6 @@ SC_Constructor(ConnectionClass *conn)
 		rv->exec_current_row = -1;
 		rv->put_data = FALSE;
 		rv->ref_CC_error = FALSE;
-		rv->lock_CC_for_rb = 0;
 		rv->join_info = 0;
 		rv->curr_param_result = 0;
 		SC_init_parse_method(rv);
@@ -446,7 +445,8 @@ SC_Constructor(ConnectionClass *conn)
 				rv, SQL_ATTR_IMP_PARAM_DESC);
 
 		rv->miscinfo = 0;
-		rv->rbonerr = 0;
+		rv->execinfo = 0;
+		rv->rb_or_tc = 0;
 		SC_reset_updatable(rv);
 		rv->diag_row_count = 0;
 		rv->stmt_time = 0;
@@ -663,7 +663,7 @@ SC_set_prepared(StatementClass *stmt, int prepared)
 					QResultClass	*res;
 					char dealloc_stmt[128];
 
-					sprintf(dealloc_stmt, "DEALLOCATE \"%s\"", stmt->plan_name);
+					SPRINTF_FIXED(dealloc_stmt, "DEALLOCATE \"%s\"", stmt->plan_name);
 					res = CC_send_query(conn, dealloc_stmt, NULL, IGNORE_ABORT_ON_CONN | ROLLBACK_ON_ERROR, NULL);
 					QR_Destructor(res);
 				}
@@ -684,18 +684,9 @@ SC_set_prepared(StatementClass *stmt, int prepared)
 RETCODE
 SC_initialize_stmts(StatementClass *self, BOOL initializeOriginal)
 {
-	ConnectionClass *conn = SC_get_conn(self);
 	ProcessedStmt *pstmt;
 	ProcessedStmt *next_pstmt;
 
-	if (self->lock_CC_for_rb > 0)
-	{
-		while (self->lock_CC_for_rb > 0)
-		{
-			LEAVE_CONN_CS(conn);
-			self->lock_CC_for_rb--;
-		}
-	}
 	if (initializeOriginal)
 	{
 		if (self->statement)
@@ -862,6 +853,7 @@ inolog("SC_clear_parse_status\n");
 	if (SC_get_Result(self))
 		SC_set_Result(self, NULL);
 	self->miscinfo = 0;
+	self->execinfo = 0;
 	/* self->rbonerr = 0; Never clear the bits here */
 
 	/*
@@ -1094,6 +1086,7 @@ SC_describe(StatementClass *self)
 mylog("              preprocess: status = READY\n");
 
 		self->miscinfo = 0;
+		self->execinfo = 0;
 
 		decideHowToPrepare(self, FALSE);
 		switch (SC_get_prepare_method(self))
@@ -1168,6 +1161,8 @@ SC_clear_error(StatementClass *self)
 		res->sqlstate[0] = '\0';
 	}
 	self->stmt_time = 0;
+	memset(&self->localtime, 0, sizeof(self->localtime));
+	self->localtime.tm_sec = -1;
 	SC_unref_CC_error(self);
 }
 
@@ -1233,7 +1228,7 @@ static const struct
 };
 
 static PG_ErrorInfo *
-SC_create_errorinfo(const StatementClass *self)
+SC_create_errorinfo(const StatementClass *self, PG_ErrorInfo *pgerror_fail_safe)
 {
 	QResultClass *res = SC_get_Curres(self);
 	ConnectionClass *conn = SC_get_conn(self);
@@ -1268,12 +1263,12 @@ SC_create_errorinfo(const StatementClass *self)
 		}
 		if (NULL != res->message)
 		{
-			strncpy_null(msg, res->message, sizeof(msg));
+			STRCPY_FIXED(msg, res->message);
 			detailmsg = resmsg = TRUE;
 		}
 		else if (NULL != res->messageref)
 		{
-			strncpy_null(msg, res->messageref, sizeof(msg));
+			STRCPY_FIXED(msg, res->messageref);
 			detailmsg = resmsg = TRUE;
 		}
 		if (msg[0])
@@ -1320,13 +1315,26 @@ SC_create_errorinfo(const StatementClass *self)
 		ermsg = msg;
 	}
 	pgerror = ER_Constructor(self->__error_number, ermsg);
-	if (!pgerror) return NULL;
+	if (!pgerror)
+	{
+		if (pgerror_fail_safe)
+		{
+			memset(pgerror_fail_safe, 0, sizeof(*pgerror_fail_safe));
+			pgerror = pgerror_fail_safe;
+			pgerror->status = self->__error_number;
+			pgerror->errorsize = sizeof(pgerror->__error_message);
+			STRCPY_FIXED(pgerror->__error_message, ermsg); 
+			pgerror->recsize = -1;
+		}
+		else
+			return NULL;
+	}
 	if (sqlstate)
-		strcpy(pgerror->sqlstate, sqlstate);
+		STRCPY_FIXED(pgerror->sqlstate, sqlstate);
 	else if (conn)
 	{
 		if (!msgend && conn->sqlstate[0])
-			strcpy(pgerror->sqlstate, conn->sqlstate);
+			STRCPY_FIXED(pgerror->sqlstate, conn->sqlstate);
 		else
 		{
 			EnvironmentClass *env = (EnvironmentClass *) CC_get_env(conn);
@@ -1337,7 +1345,7 @@ SC_create_errorinfo(const StatementClass *self)
 			{
 				errornum = 1 - LOWEST_STMT_ERROR;
 			}
-			strcpy(pgerror->sqlstate, EN_is_odbc3(env) ?
+			STRCPY_FIXED(pgerror->sqlstate, EN_is_odbc3(env) ?
 				   Statement_sqlstate[errornum].ver3str :
 				   Statement_sqlstate[errornum].ver2str);
 		}
@@ -1430,7 +1438,7 @@ inolog("SC_set_error_from_res %p->%p check=%i\n", from_res ,self, check);
 			repstate = TRUE;
 	}
 	if (repstate)
-		strcpy(self_res->sqlstate, from_res->sqlstate);
+		STRCPY_FIXED(self_res->sqlstate, from_res->sqlstate);
 }
 
 void
@@ -1478,7 +1486,7 @@ inolog("SC_error_copy %p->%p check=%i\n", from ,self, check);
 			repstate = TRUE;
 	}
 	if (repstate)
-		strcpy(self_res->sqlstate, from_res->sqlstate);
+		STRCPY_FIXED(self_res->sqlstate, from_res->sqlstate);
 }
 
 
@@ -1505,7 +1513,7 @@ inolog("SC_full_error_copy %p->%p\n", from ,self);
 	}
 	else if (!allres)
 		return;
-	pgerror = SC_create_errorinfo(from);
+	pgerror = SC_create_errorinfo(from, NULL);
 	if (!pgerror || !pgerror->__error_message[0])
 	{
 		ER_Destructor(pgerror);
@@ -1528,10 +1536,18 @@ PGAPI_StmtError(SQLHSTMT	hstmt,
 				UWORD flag)
 {
 	/* CC: return an error of a hdesc  */
+	PG_ErrorInfo		*pgerror, error;
 	StatementClass *stmt = (StatementClass *) hstmt;
+	int errnum = SC_get_errornumber(stmt);
 
-	stmt->pgerror = SC_create_errorinfo(stmt);
-	return ER_ReturnError(&(stmt->pgerror), RecNumber, szSqlState,
+	if (pgerror = SC_create_errorinfo(stmt, &error), NULL == pgerror)
+		return SQL_NO_DATA_FOUND;
+	if (pgerror != &error)
+		stmt->pgerror = pgerror;
+	if (STMT_NO_MEMORY_ERROR == errnum &&
+	    !pgerror->__error_message[0])
+		STRCPY_FIXED(pgerror->__error_message, "Memory Allocation Error??");
+	return ER_ReturnError(pgerror, RecNumber, szSqlState,
 						  pfNativeError, szErrorMsg, cbErrorMsgMax,
 						  pcbErrorMsg, flag);
 }
@@ -1546,6 +1562,25 @@ SC_get_time(StatementClass *stmt)
 	return stmt->stmt_time;
 }
 
+struct tm *
+SC_get_localtime(StatementClass *stmt)
+{
+	struct tm * tim;
+
+	if (stmt->localtime.tm_sec < 0)
+	{
+		SC_get_time(stmt);
+#ifdef HAVE_LOCALTIME_R
+		tim = localtime_r(&stmt->stmt_time, &(stmt->localtime));
+#else
+		tim = localtime(&stmt->stmt_time);
+		stmt->localtime = *tim;
+#endif /* HAVE_LOCALTIME_R */
+	}
+
+	return &(stmt->localtime);
+}
+
 RETCODE
 SC_fetch(StatementClass *self)
 {
@@ -1729,6 +1764,11 @@ inolog("curt=%d\n", curt);
 					result = SQL_SUCCESS_WITH_INFO;
 					break;
 
+				case COPY_INVALID_STRING_CONVERSION:    /* invalid string */
+					SC_set_error(self, STMT_STRING_CONVERSION_ERROR, "invalid string conversion occured.", func);
+					result = SQL_ERROR;
+					break;
+
 					/* error msg already filled in */
 				case COPY_GENERAL_ERROR:
 					result = SQL_ERROR;
@@ -1802,7 +1842,7 @@ SC_execute(StatementClass *self)
 	}
 	/* issue BEGIN ? */
 	issue_begin = TRUE;
-	if (self->internal)
+	if (!self->external)
 		issue_begin = FALSE;
 	else if (is_in_trans)
 	{
@@ -1850,7 +1890,7 @@ SC_execute(StatementClass *self)
 	{
 		char query[64];
 
-		snprintf(query, sizeof(query), "SET statement_timeout = %d",
+		SPRINTF_FIXED(query, "SET statement_timeout = %d",
 				 (int) self->options.stmt_timeout * 1000);
 		res = CC_send_query(conn, query, NULL, 0, NULL);
 		if (QR_command_maybe_successful(res))
@@ -1890,7 +1930,6 @@ SC_execute(StatementClass *self)
 			if (SC_get_errornumber(self) <= 0)
 			{
 				SC_set_error(self, STMT_NO_RESPONSE, "Could not receive the response, communication down ??", func);
-				CC_on_abort(conn, CONN_DEAD);
 			}
 			goto cleanup;
 		}
@@ -1910,11 +1949,12 @@ SC_execute(StatementClass *self)
 			qi.result_in = NULL;
 			qi.cursor = SC_cursor_name(self);
 			qi.fetch_size = qi.row_size = ci->drivers.fetch_max;
-			snprintf(fetch, sizeof(fetch),
+			SPRINTF_FIXED(fetch,
 					 "fetch " FORMAT_LEN " in \"%s\"",
 					 qi.fetch_size, SC_cursor_name(self));
 			qryi = &qi;
 			appendq = fetch;
+			qflag &= (~READ_ONLY_QUERY); /* must be a SAVEPOINT after DECLARE */
 		}
 		res = SC_get_Result(self);
 		if (self->curr_param_result && res)
@@ -1988,7 +2028,7 @@ SC_execute(StatementClass *self)
 		{
 			if (!is_in_trans)
 				CC_set_in_manual_trans(conn);
-			if (!self->internal && CC_does_autocommit(conn))
+			if (self->external && CC_does_autocommit(conn))
 				CC_commit(conn);
 		}
 	}
@@ -2004,10 +2044,10 @@ SC_execute(StatementClass *self)
 		was_ok = QR_command_successful(res);
 		was_nonfatal = QR_command_nonfatal(res);
 
-		if (was_ok)
-			SC_set_errornumber(self, STMT_OK);
-		else if (0 < SC_get_errornumber(self))
+		if (0 < SC_get_errornumber(self))
 			;
+		else if (was_ok)
+			SC_set_errornumber(self, STMT_OK);
 		else if (was_nonfatal)
 			SC_set_errornumber(self, STMT_INFO_ONLY);
 		else
@@ -2136,8 +2176,8 @@ inolog("!!! numfield=%d field_type=%u\n", QR_NumResultCols(res), QR_get_field_ty
 			qi.result_in = NULL;
 			qi.cursor = SC_cursor_name(self);
 			qi.cache_size = qi.row_size = ci->drivers.fetch_max;
-			snprintf(fetch, sizeof(fetch), "fetch " FORMAT_LEN " in \"%s\"", qi.fetch_size, SC_cursor_name(self));
-			res = CC_send_query(conn, fetch, &qi, qflag, SC_get_ancestor(self));
+			SPRINTF_FIXED(fetch, "fetch " FORMAT_LEN " in \"%s\"", qi.fetch_size, SC_cursor_name(self));
+			res = CC_send_query(conn, fetch, &qi, qflag | READ_ONLY_QUERY, SC_get_ancestor(self));
 			if (NULL != res)
 				SC_set_Result(self, res);
 		}
@@ -2290,7 +2330,7 @@ SC_log_error(const char *func, const char *desc, const StatementClass *self)
 		{
 			qlog("                 ------------------------------------------------------------\n");
 			qlog("                 hdbc=%p, stmt=%p, result=%p\n", self->hdbc, self, res);
-			qlog("                 prepare=%d, internal=%d\n", self->prepare, self->internal);
+			qlog("                 prepare=%d, external=%d\n", self->prepare, self->external);
 			qlog("                 bindings=%p, bindings_allocated=%d\n", opts->bindings, opts->allocated);
 			qlog("                 parameters=%p, parameters_allocated=%d\n", apdopts->parameters, apdopts->allocated);
 			qlog("                 statement_type=%d, statement='%s'\n", self->statement_type, NULLCHECK(self->statement));
@@ -2329,6 +2369,7 @@ static BOOL
 RequestStart(StatementClass *stmt, ConnectionClass *conn, const char *func)
 {
 	BOOL	ret = TRUE;
+	unsigned int	svpopt = 0;
 
 #ifdef	_HANDLE_ENLIST_IN_DTC_
 	if (conn->asdum)
@@ -2339,13 +2380,15 @@ RequestStart(StatementClass *stmt, ConnectionClass *conn, const char *func)
 		SC_set_error(stmt, STMT_COMMUNICATION_ERROR, "The connection has been lost", __FUNCTION__);
 		return SQL_ERROR;
 	}
-	if (SC_accessed_db(stmt))
+	if (CC_started_rbpoint(conn))
 		return TRUE;
-	if (SQL_ERROR == SetStatementSvp(stmt))
+	if (SC_is_readonly(stmt))
+		svpopt |= SVPOPT_RDONLY;
+	if (SQL_ERROR == SetStatementSvp(stmt, svpopt))
 	{
 		char	emsg[128];
 
-		snprintf(emsg, sizeof(emsg), "internal savepoint error in %s", func);
+		SPRINTF_FIXED(emsg, "internal savepoint error in %s", func);
 		SC_set_error_if_not_set(stmt, STMT_INTERNAL_ERROR, emsg, func);
 		return FALSE;
 	}
@@ -2381,19 +2424,20 @@ libpq_bind_and_exec(StatementClass *stmt)
 	QResultClass *res = NULL;
 	char	   *cmdtag;
 	char	   *rowcount;
-	BOOL		ret = FALSE;
 
 	if (!RequestStart(stmt, conn, func))
 		return NULL;
 
-	if (CC_is_in_trans(conn) && !SC_accessed_db(stmt))
+#ifdef	NOT_USED
+	if (CC_is_in_trans(conn) && !CC_started_rbpoint(conn))
 	{
-		if (SQL_ERROR == SetStatementSvp(stmt))
+		if (SQL_ERROR == SetStatementSvp(stmt, 0))
 		{
 			SC_set_error_if_not_set(stmt, STMT_INTERNAL_ERROR, "internal savepoint error in build_libpq_bind_params", func);
 			return NULL;
 		}
 	}
+#endif /* NOT_USED */
 
 	/* 1. Bind */
 	mylog("%s: bind stmt=%p\n", func, stmt);
@@ -2404,6 +2448,8 @@ libpq_bind_and_exec(StatementClass *stmt)
 								 &paramLengths, &paramFormats,
 								 &resultFormat))
 	{
+		if (SC_get_errornumber(stmt) <= 0)
+			SC_set_errornumber(stmt, STMT_NO_MEMORY_ERROR);
 		goto cleanup;
 	}
 
@@ -2525,6 +2571,7 @@ inolog("get_Result=%p %p %d\n", res, SC_get_Result(stmt), stmt->curr_param_resul
 		case PGRES_COPY_BOTH:
 		default:
 			/* skip the unexpected response if possible */
+			QR_set_rstatus(res, PORES_BAD_RESPONSE);
 			CC_set_error(conn, CONNECTION_BACKEND_CRAZY, "Unexpected protocol character from backend (send_query)", func);
 			CC_on_abort(conn, CONN_DEAD);
 
@@ -2535,8 +2582,6 @@ inolog("get_Result=%p %p %d\n", res, SC_get_Result(stmt), stmt->curr_param_resul
 	if (res != newres && NULL != newres)
 		QR_Destructor(newres);
 
-	ret = TRUE;
-
 cleanup:
 	if (pgres)
 		PQclear(pgres);
@@ -2557,10 +2602,7 @@ cleanup:
 	if (paramFormats)
 		free(paramFormats);
 
-	if (ret)
-		return res;
-	else
-		return NULL;
+	return res;
 }
 
 /*
@@ -2635,7 +2677,10 @@ mylog("sta_pidx=%d end_pidx=%d num_p=%d\n", sta_pidx, end_pidx, num_params);
 
 		paramTypes = malloc(sizeof(Oid) * num_params);
 		if (paramTypes == NULL)
+		{
+			SC_set_errornumber(stmt, STMT_NO_MEMORY_ERROR);
 			goto cleanup;
+		}
 
 		mylog("ipdopts->allocated: %d\n", ipdopts->allocated);
 		j = 0;
